/**
 * Copyright (c) 2013-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.so;

import static org.junit.Assert.assertEquals;

import java.util.Properties;

import org.junit.BeforeClass;
import org.junit.Test;

/**
 * Unit tests of {@link NagiosMapper}.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class NagiosMapperTest {

    private final static String DEFECT_EMITTER_1 = "localhost";

    private final static String NAGIOS_HOST_1 = "petals-esb-1";

    private final static String DEFECT_NAME_1 = "defect.1";

    private final static String NAGIOS_SERVICE_1 = "petals-esb-service-1";

    private final static Properties defaultMappingRules = new Properties();

    @BeforeClass
    public static void setup() {
        defaultMappingRules.setProperty(NagiosMapper.EMITTER_PREFIX + DEFECT_EMITTER_1,
                NAGIOS_HOST_1);
        defaultMappingRules.setProperty(NagiosMapper.NAME_PREFIX + DEFECT_NAME_1, NAGIOS_SERVICE_1);
    }

    /**
     * <p>
     * Check to get a Nagios host from a defect emitter when the mapping rule
     * set is <code>null</code>.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>the defect emitter is returned as Nagios host.</li>
     * </ul>
     * 
     */
    @Test
    public void testGetNagiosHost_MappinRulesNull() {

        final NagiosMapper mapper = new NagiosMapper(null);
        final String defectEmitter = "defectEmitter";

        assertEquals("Unexpected Nagios host", defectEmitter, mapper.getNagiosHost(defectEmitter));
    }

    /**
     * <p>
     * Check to get a Nagios host from a defect emitter when the mapping rule
     * set is empty.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>the defect emitter is returned as Nagios host.</li>
     * </ul>
     * 
     */
    @Test
    public void testGetNagiosHost_MappinRulesEmpty() {

        final NagiosMapper mapper = new NagiosMapper(new Properties());
        final String defectEmitter = "defectEmitter";

        assertEquals("Unexpected Nagios host", defectEmitter, mapper.getNagiosHost(defectEmitter));
    }

    /**
     * <p>
     * Check to get a Nagios host from a defect emitter when the mapping rule
     * set is filled, but not contain the defect emitter.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>the defect emitter is returned as Nagios host.</li>
     * </ul>
     * 
     */
    @Test
    public void testGetNagiosHost_MappinRulesMissing() {

        final NagiosMapper mapper = new NagiosMapper(defaultMappingRules);
        final String defectEmitter = "missing-defectEmitter";

        assertEquals("Unexpected Nagios host", defectEmitter, mapper.getNagiosHost(defectEmitter));
    }

    /**
     * <p>
     * Check to get a Nagios host from a defect emitter when the mapping rule
     * set is filled with the expected defect emitter.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>the right Nagios host is returned.</li>
     * </ul>
     * 
     */
    @Test
    public void testGetNagiosHost_MappinRules() {

        final NagiosMapper mapper = new NagiosMapper(defaultMappingRules);

        assertEquals("Unexpected Nagios host", NAGIOS_HOST_1,
                mapper.getNagiosHost(DEFECT_EMITTER_1));
    }

    /**
     * <p>
     * Check to get a Nagios service from a defect name when the mapping rule
     * set is <code>null</code>.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>the defect name is returned as Nagios service.</li>
     * </ul>
     * 
     */
    @Test
    public void testGetNagiosService_MappinRulesNull() {

        final NagiosMapper mapper = new NagiosMapper(null);
        final String defectName = "defectName";

        assertEquals("Unexpected Nagios service", defectName, mapper.getNagiosService(defectName));
    }

    /**
     * <p>
     * Check to get a Nagios service from a defect name when the mapping rule
     * set is empty.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>the defect name is returned as Nagios service.</li>
     * </ul>
     * 
     */
    @Test
    public void testGetNagiosService_MappinRulesEmpty() {

        final NagiosMapper mapper = new NagiosMapper(new Properties());
        final String defectName = "defectName";

        assertEquals("Unexpected Nagios service", defectName, mapper.getNagiosService(defectName));
    }

    /**
     * <p>
     * Check to get a Nagios service from a defect name when the mapping rule
     * set is filled, but not contain the defect name.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>the defect name is returned as Nagios service.</li>
     * </ul>
     * 
     */
    @Test
    public void testGetNagiosService_MappinRulesMissing() {

        final NagiosMapper mapper = new NagiosMapper(defaultMappingRules);
        final String defectName = "missing-defectName";

        assertEquals("Unexpected Nagios service", defectName, mapper.getNagiosService(defectName));
    }

    /**
     * <p>
     * Check to get a Nagios service from a defect name when the mapping rule
     * set is filled with the expected defect name.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>the right Nagios service is returned.</li>
     * </ul>
     * 
     */
    @Test
    public void testGetNagiosService_MappinRules() {

        final NagiosMapper mapper = new NagiosMapper(defaultMappingRules);

        assertEquals("Unexpected Nagios service", NAGIOS_SERVICE_1,
                mapper.getNagiosService(DEFECT_NAME_1));
    }

}
