/**
 * Copyright (c) 2013-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.so.components.framework;

import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.junit.Test;
import org.ow2.petals.cli.extension.command.monitoring.so.api.EmittableDefect;
import org.ow2.petals.cli.extension.command.monitoring.so.api.SubscriptionObject;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * Unit tests of {@link AbstractMonitoredComponent}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public abstract class AbstractMonitoredComponentTest {

    /**
     * <p>
     * Check {@link AbstractMonitoredComponent#getEmittableDefects()} against a
     * dedicated emittable defect list set to <code>null</code>.
     * </p>
     * <p>
     * Expected result: only the emittable defects provided by the CDK are
     * available.
     * </p>
     */
    @Test
    public void testGetEmittableDefects_NullDedicatedDefects() {
        final SubscriptionObject so = new AbstractMonitoredComponent() {

            @Override
            public String getName() {
                return null;
            }

            @Override
            public String getDescription() {
                return null;
            }

            @Override
            public List<EmittableDefect> getDedicatedEmittableDefects() {
                // No dedicated emittable defects
                return null;
            }
            
        };

        assertEquals("unexpected sub-functions",
                Arrays.asList((EmittableDefect[]) ReflectionHelper
                .getPrivateFieldValue(AbstractMonitoredComponent.class, null,
                        "CDK_EMITTABLE_DEFECTS")), so.getEmittableDefects());
    }

    /**
     * <p>
     * Check {@link AbstractMonitoredComponent#getEmittableDefects()} against an
     * empty dedicated emittable defect list.
     * </p>
     * <p>
     * Expected result: only the emittable defects provided by the CDK are
     * available.
     * </p>
     */
    @Test
    public void testGetEmittableDefects_EmptyDedicatedDefects() {
        final SubscriptionObject so = new AbstractMonitoredComponent() {

            @Override
            public String getName() {
                return null;
            }

            @Override
            public String getDescription() {
                return null;
            }

            @Override
            public List<EmittableDefect> getDedicatedEmittableDefects() {
                // No dedicated emittable defects
                return Collections.emptyList();
            }

        };

        assertEquals("unexpected sub-functions", Arrays.asList((EmittableDefect[]) ReflectionHelper
                .getPrivateFieldValue(AbstractMonitoredComponent.class, null,
                        "CDK_EMITTABLE_DEFECTS")), so.getEmittableDefects());
    }

    /**
     * <p>
     * Check {@link AbstractMonitoredComponent#getEmittableDefects()} against a
     * dedicated emittable defect list set to a valid value.
     * </p>
     * <p>
     * Expected result: the emittable defects list is composed of both cdk defects and
     * dedicated defects.
     * </p>
     */
    @Test
    public void testGetEmittableDefects_DedicatedDefects() {
        final EmittableDefect dedicatedEmittableDefect = new EmittableDefect() {

            @Override
            public String getName() {
                return null;
            }

            @Override
            public String getDescription() {
                return null;
            }
        };

        final SubscriptionObject so = new AbstractMonitoredComponent() {

            @Override
            public String getName() {
                return null;
            }

            @Override
            public String getDescription() {
                return null;
            }

            @Override
            public List<EmittableDefect> getDedicatedEmittableDefects() {
                // No dedicated emittable defects
                return Arrays.asList(new EmittableDefect[] { dedicatedEmittableDefect });
            }

        };

        final List<EmittableDefect> expectedEmittableDefects = new ArrayList<EmittableDefect>();
        expectedEmittableDefects.addAll(Arrays.asList((EmittableDefect[]) ReflectionHelper
                .getPrivateFieldValue(AbstractMonitoredComponent.class, null,
                        "CDK_EMITTABLE_DEFECTS")));
        expectedEmittableDefects.add(dedicatedEmittableDefect);

        assertEquals("unexpected sub-functions", expectedEmittableDefects, so.getEmittableDefects());
    }

}
